### First case study ##############################################################################

  ### Date: 10 March 2017

  ### Description: 
  ###
  ### Code for calculating the number of episodes of abnormal functioning of the liver
  ### in patients with liver cirrhosis from a clinical trial from Copenhagen. The
  ### treatment group received the hormone prednisone (n=251), while controls received
  ### a placebo (n=237). The main question of the analysis is whether prednisone
  ### prolongs survival and how it affects liver functioning. 
  ###
  ### The data set was originally analysed by Andersen et al [1] and subsequently was
  ### included in many other studies. Recently, the data set was made available in
  ### the mstate package [2]. 
  ###
  ### References:
  ### [1]   P. K. Andersen, . Borgan, R. D. Gill, and N. Keiding. Statistical Models Based on
  ###       Counting Processes. Springer, 2nd edition, 1993.
  ### [2]   S. E. Hardy, J. A. Dubin, T. R. Holford, and T. M. Gill. Transitions between
  ###       states of disability and independence among older persons. American Journal of
  ###       Epidemiology, 161(6):575-584, 2005



### Load packages and data ########################################################################

  # Packages
  library(mstate)
  library(Matrix)

  # Data
  data(prothr)
  
  
  
### Calculate rates for placebo group #############################################################
  
  # Length of spells in days
  prothr$length <- prothr$Tstop - prothr$Tstart
  
  # Exposures
  normal.exposure <- sum(prothr$length[prothr$from==1&prothr$treat=="Placebo"]) 
  low.exposure <- sum(prothr$length[prothr$from==2&prothr$treat=="Placebo"]) 
  
  #Transitions from normal (Placebo)
  normal.transitions <- c(
                          0,
                          sum(prothr$from==1&prothr$to==2&prothr$status==1&prothr$treat=="Placebo"),
                          sum(prothr$from==1&prothr$to==3&prothr$status==1&prothr$treat=="Placebo")
                          )
  
  # Transitions from low (Placebo)
  low.transitions <- c(
                       sum(prothr$from==2&prothr$to==1&prothr$status==1&prothr$treat=="Placebo"),
                       0,
                       sum(prothr$from==2&prothr$to==3&prothr$status==1&prothr$treat=="Placebo")
                       )
  
  # 'Transitions' from death 
  death <- c(0,0,0)
  
  # Matrix of transition rates (Placebo)
  Tm.placebo <- rbind(normal.transitions/normal.exposure,
              low.transitions/low.exposure,
              death)
  
  # Transform into probabilities (Placebo)
  M <- diag(rowSums(Tm.placebo-diag(diag(Tm.placebo)))) +  - (Tm.placebo-diag(diag(Tm.placebo)))
  m <- dim(M)[1]
  P.placebo <- (diag(1,m)-1/2*M)%*%(solve(diag(1,m)+1/2*M ))
  rownames(P.placebo) <- c("Normal","Low","Dead")
  colnames(P.placebo) <- rownames(P.placebo)
  
  
  
### Calculate rates for treatment group ###########################################################
  
  # Exposures
  normal.exposure <- sum(prothr$length[prothr$from==1&prothr$treat=="Prednisone"]) 
  low.exposure <- sum(prothr$length[prothr$from==2&prothr$treat=="Prednisone"]) 
  
  # Transitions from normal (Prednisone)
  
  normal.transitions <- c(
    0,
    sum(prothr$from==1&prothr$to==2&prothr$status==1&prothr$treat=="Prednisone"),
    sum(prothr$from==1&prothr$to==3&prothr$status==1&prothr$treat=="Prednisone")
  )
  
  # Transitions from low (Prednisone)
  low.transitions <- c(
    sum(prothr$from==2&prothr$to==1&prothr$status==1&prothr$treat=="Prednisone"),
    0,
    sum(prothr$from==2&prothr$to==3&prothr$status==1&prothr$treat=="Prednisone")
  )
  
  # Matrix of transition rates (Prednisone)
  Tm.treatment <- rbind(normal.transitions/normal.exposure,
                      low.transitions/low.exposure,
                      death)
  
  # Transform into probabilities (Prednisone)
  M <- diag(rowSums(Tm.treatment-diag(diag(Tm.treatment)))) +  - (Tm.treatment-diag(diag(Tm.treatment)))
  m <- dim(M)[1]
  P.treatment <- (diag(1,m)-1/2*M)%*%(solve(diag(1,m)+1/2*M ))  
  rownames(P.treatment) <- rownames(P.placebo)
  colnames(P.treatment) <- rownames(P.placebo)
  
  
  
### Build matrices for 10 years ###################################################################
  
  # Total number of days; could be changed to, e.g., 365*5, to use 5 years instead of 10 years
  days <- 365*10
  
  # Number of (transient) states
  m <- dim(P.treatment)[1]
  n <- sum(rowSums(P.treatment-diag(diag(P.treatment)))>0)
  
  # Matrix U
  U.treatment <- P.treatment[1:n,1:n]
  U.placebo <- P.placebo[1:n,1:n]
  
  # Probabilities of dying
  death.treatment <- P.treatment[1:n,(n+1):m]
  death.placebo <- P.placebo[1:n,(n+1):m]
  
  # Empty matrices, will be filled with probabilities below
  tmp.treatment <- matrix(data=0,nrow=days*n+(m-n),ncol=days*n+(m-n))
  tmp.placebo <- matrix(data=0,nrow=days*n+(m-n),ncol=days*n+(m-n))
  
  # Row and column names for above matrices
  rownames(tmp.treatment) <- c(rep(c("Normal","Low"),days),"Death")
  colnames(tmp.treatment) <- rownames(tmp.treatment)
  colnames(tmp.placebo) <- colnames(tmp.treatment)
  rownames(tmp.placebo) <- rownames(tmp.treatment)
  
  # For each time step, transition probabiblities will be filled into matrix
  for(i in 1:(days-1)) {
    tmp.treatment[((i-1)*n+1):((i-1)*n+n),(i*n+1):(i*n+n)] <- U.treatment
    tmp.treatment[((i-1)*n+1):((i-1)*n+n),((days*n)+1):(days*n+(m-n))] <- death.treatment
  }
  
  # Set some probabilities to 0 or 1 
  tmp.treatment[(days*n-(n-1)):(days*n),(days*n-(n-1)):(days*n)] <- 0 # After 'days' no transitions to transient states
  tmp.treatment[(days*n-(n-1)):(days*n),"Death"] <- 1 # After 'days' death is certain
  tmp.treatment["Death","Death"] <- 1 # Death is absorbing
  
  # For each time step, transition probabiblities will be filled into matrix
  for(i in 1:(days-1)) {
    tmp.placebo[((i-1)*n+1):((i-1)*n+n),(i*n+1):(i*n+n)] <- U.placebo
    tmp.placebo[((i-1)*n+1):((i-1)*n+n),((days*n)+1):(days*n+(m-n))] <- death.placebo
  }
  
  # Set some probabilities to 0 or 1
  tmp.placebo[(days*n-(n-1)):(days*n),(days*n-(n-1)):(days*n)] <- 0
  tmp.placebo[(days*n-(n-1)):(days*n),"Death"] <- 1
  tmp.placebo["Death","Death"] <- 1
  
  # Save matrices in class of Matrix package; more efficient
  Pm.treatment <- Matrix(tmp.treatment)
  Pm.placebo <- Matrix(tmp.placebo)
  
  
  
### Time spend in state ###########################################################################
  
  # Number of (transient) states
  m <- dim(Pm.treatment)[1]
  n <- sum(rowSums(Pm.treatment-diag(diag(Pm.treatment)))>0)
  
  # Matrix N as per equation (1) in the text
  N.placebo <- solve(diag(1,n)-Pm.placebo[1:n,1:n])
  N.treatment <- solve(diag(1,n)-Pm.treatment[1:n,1:n])
  
  # Row sums of matrix N
  n.placebo.normal <- rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Normal")])
  n.placebo.low <-rowSums(as.matrix(N.placebo)[1:2,which(colnames(Pm.placebo)=="Low")])
  n.treatment.normal <- rowSums(as.matrix(N.treatment)[1:2,which(colnames(Pm.treatment)=="Normal")])
  n.treatment.low <- rowSums(as.matrix(N.treatment)[1:2,which(colnames(Pm.treatment)=="Low")])
  
  
  
### Episodes spend in state #######################################################################
  
  # Matrix of rewards R, following equation (3)
  R <- matrix(data=0,nrow=m,ncol=m)
  R[ which(rownames(Pm.treatment)=="Low"),which(colnames(Pm.treatment)%in%c("Normal"))] <- 1
  R <- Matrix(R)
  
  # Matrix Z as needed in equation (2)
  Z <- cbind(diag(1,n),matrix(data=0,ncol=m-n,nrow=n))
  Z <- Matrix(Z)
  
  # Vector e as given by equation (2)
  e.treatment <- (N.treatment%*%Z%*%(Pm.treatment*R)%*%rep(1,m))[1:2]
  e.placebo <- (N.placebo%*%Z%*%(Pm.placebo*R)%*%rep(1,m))[1:2]
  
  
  
### Table for paper ###############################################################################
  
  # Load results from bootstrapping
  load("U:/Tmp/markov_spells_prednisone_bootstrap.rda")
  
  # Edit results from bootstrapping
  normal.placebo <- unlist(lapply(results,function(x) x[[2]]))
  normal.treatment <- unlist(lapply(results,function(x) x[[6]]))
  low.placebo <- unlist(lapply(results,function(x) x[[4]]))
  low.treatment <- unlist(lapply(results,function(x) x[[8]]))
  life.placebo <- normal.placebo+low.placebo
  life.treatment <- normal.treatment+low.treatment
  episodes.placebo <- unlist(lapply(results,function(x) x[[12]]))
  episodes.treatment <- unlist(lapply(results,function(x) x[[10]]))
  
  # First column of Table 1 (control/placebo)
  round(n.placebo.low[2]/365,digits=1)
  round(quantile(low.placebo/365,probs=c(0.025,0.975)),digits=1)
  round(n.placebo.normal[2]/365,digits=1)
  round(quantile(normal.placebo/365,probs=c(0.025,0.975)),digits=1)
  round(n.placebo.low[2]/365+n.placebo.normal[2]/365,digits=1)
  round(quantile(life.placebo/365,probs=c(0.025,0.975)),digits=1)
  round(e.placebo[2],digits=1)
  round(quantile(episodes.placebo,probs=c(0.025,0.975)),digits=1)
  round((n.placebo.normal[2]/e.placebo[2])/365,digits=1)
  round(quantile((normal.placebo/365)/episodes.placebo,probs=c(0.025,0.975)),digits=1)
  
  # Second column of Table 1 (treatment/prednisone)
  round(n.treatment.low[2]/365,digits=1)
  round(quantile(low.treatment/365,probs=c(0.025,0.975)),digits=1)
  round(n.treatment.normal[2]/365,digits=1)
  round(quantile(normal.treatment/365,probs=c(0.025,0.975)),digits=1)
  round(n.treatment.low[2]/365+n.treatment.normal[2]/365,digits=1)
  round(quantile(life.treatment/365,probs=c(0.025,0.975)),digits=1)
  round(e.treatment[2],digits=1)
  round(quantile(episodes.treatment,probs=c(0.025,0.975)),digits=1)
  round((n.treatment.normal[2]/e.treatment[2])/365,digits=1)
  round(quantile((normal.treatment/365)/episodes.treatment,probs=c(0.025,0.975)),digits=1)
  