### Numerical example

  # Transition matrix
  P <- matrix(data=c(0.0,0.5,0.5,0,
                     0.0,0.0,0.0,1,
                     0.0,0.0,0.0,1,
                     0.0,0.0,0.0,1),byrow=T,ncol=4)
  
  # Transition matrix without absorbing states
  U <- P[-4,-4]
  
  # Equation (1)
  I <- diag(1,3)
  N <- solve(I-U)
  
  # Equation (2)
  Z <- cbind(I,rep(0,3))
  R <- matrix(data=c(0,0,1,0,
                     0,0,0,0,
                     0,0,0,0,
                     0,0,0,0),byrow=T,ncol=4)
  e <- N%*%Z%*%(P*R)%*%rep(1,4)
  
  # Equation (4)
  N[1,2]/e[1,1]

  
  
### Simulation example (1)  
  
  # Names of states
  states <- c("Healthy","Unhealthy","Dead")
  
  # Transition matrix
  P <- matrix(data=c(0.80,0.15,0.05,
                     0.10,0.80,0.10,
                     0.00,0.00,1.00),
              byrow=T,ncol=3,
              dimnames=list(states,states))

  # Create Markov chain
  library(markovchain)
  mcHealth <- new("markovchain",states=states,
                  transitionMatrix=P,name="Health")  
  
  # Number of simulation runs
  nsim <- 250000
  
  # Number of simulation steps
  nstep <- 500
  
  # For results
  sequence_no <- numeric(nsim)
  sequence_ln <- numeric(nsim)
  
  # Set seed
  set.seed(seed=294)
  
  # Run simulations - might take a few minutes
  for(sim in 1:nsim) {
    
    # Control
    if(sim%%10000==0) cat(".")
  
    # Simulate sequence
    simulation <- rmarkovchain(n=nstep,object=mcHealth,t0="Healthy",include.t0=T)
    simulation <- as.numeric(simulation=="Unhealthy")
    
    # Assign results
    sequence_ln[sim] <- sum(simulation)
    sequence_no[sim] <- sum(diff(simulation)==1)
  
  }
  
  # Total result
  sim_result <- mean(sequence_ln[sequence_no>0]/sequence_no[sequence_no>0])
  
  # Analytical solution
  U <- P[-3,-3]
  I <- diag(1,2)
  N <- solve(I-U)
  Z <- cbind(I,rep(0,2))
  R <- matrix(data=c(0,1,0,
                     0,0,0,
                     0,0,0),byrow=T,ncol=3)
  e <- N%*%Z%*%(P*R)%*%rep(1,3)
  
  # Equation (4)
  analytical_result <- N[1,2]/e[1,1]
  
  # Compare
  sim_result
  analytical_result
  
  # Difference is small: absolute
  sim_result - analytical_result
  
  # Relative percentage error: Erorr is less than 0.01%
  (1 - sim_result / analytical_result)*100
  
  
  
### Simulation example (2)  
  
  # Names of states
  states <- c("Healthy","Mild","Severe","Dead")
  
  # Transition matrix
  P <- matrix(data=c(0.85,0.09,0.01,0.05,
                     0.15,0.70,0.10,0.05,
                     0.05,0.10,0.75,0.10,
                     0.00,0.00,0.00,1.00),
              byrow=T,ncol=4,
              dimnames=list(states,states))
  
  # Create Markov chain
  mcHealth <- new("markovchain",states=states,
                  transitionMatrix=P,name="Health")  
  
  # Number of simulation runs
  nsim <- 250000
  
  # Number of simulation steps
  nstep <- 1000
  
  # For results
  sequence_no <- numeric(nsim)
  sequence_ln <- numeric(nsim)
  
  # Set seed
  set.seed(seed=94)
  
  # Run simulations - might take a few minutes
  for(sim in 1:nsim) {
    
    # Control
    if(sim%%10000==0) cat(".")
    
    # Simulate sequence
    simulation <- rmarkovchain(n=nstep,object=mcHealth,t0="Healthy",include.t0=T)
    simulation <- as.numeric(simulation=="Severe")
    
    # Assign results
    sequence_ln[sim] <- sum(simulation)
    sequence_no[sim] <- sum(diff(simulation)==1)
    
  }
  
  # Total result
  sim_result <- mean(sequence_ln[sequence_no>0]/sequence_no[sequence_no>0])
  
  # Analytical solution
  U <- P[-4,-4]
  I <- diag(1,3)
  N <- solve(I-U)
  Z <- cbind(I,rep(0,3))
  R <- matrix(data=c(0,0,1,0,
                     0,0,1,0,
                     0,0,0,0,
                     0,0,0,0),byrow=T,ncol=4)
  e <- N%*%Z%*%(P*R)%*%rep(1,4)
  
  # Equation (4)
  analytical_result <- N[1,3]/e[1,1]
  
  # Compare
  sim_result
  analytical_result
  
  # Difference is small: absolute
  sim_result - analytical_result
  
  # Relative percentage error: Erorr is less than 0.01%
  (1 - sim_result / analytical_result)*100
  